function rldesign(G,sldG)
// GUI for Controller Design Using Root Locus
//
//    Copyright 2011 Trity Technologies.
//    
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 2 of the License, or
//    (at your option) any later version.
//    
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//    
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/
//
// Calling Sequence
//     rldesign
//
// Description
//    This GUI is used to design a controller using root locus
//    
// See also
//     sciIdent
//     IP_GUI
//
// Authors
//     Tan C.L.  

// This is rldesign 3.1, Scicoslab 4.4.1 version
// Changes in 3.1 6/6/2011 : Fix bug in evans gain
// Change in 3.2 31/5/2012 : Change Evans Root Locus to auto range
// Changes in 4.0 4/4/2013 : Evans Root Locus range base on dcgain, step response autorange
// Changes in 4.1 1/4/2014 : Slider change range when reach limits
//  : check children existence before del
//  : improve editbox functions

ieee(2);
funcprot(0);
//clear;
//exec('bode2.sci');
//global handles.axes1_h handles.axes2_h handles.axes3_h handles.axes4_h  handles.g;
//sleep(1000);
x = 0; y = 0 ;
handles.axes1  = 1;
handles.axes2  = 1;
handles.axes3  = 1;
handles.axes4  = 1;
h=figure('position',[200 50 1000 550]);


// Modified on 14-4-2011 
[lhs,rhs] = argn();

if rhs == 0
    Gnum = "3";
    Gden = "3*s^2 + s + 2";
    handles.sldG = 0.1;
elseif rhs == 1
    Gnum = pol2str(G.num);
    Gden = pol2str(G.den);
    handles.sldG = 0.1;
elseif rhs == 2
    Gnum = pol2str(G.num);
    Gden = pol2str(G.den);
    handles.sldG = sldG;
end    

//*************
//Initialize Axes
//*************
subplot(4,1,1);
handles.axes1=gca();
handles.axes1.axes_bounds = [0.15 -0.1 0.42,1.2];
plot(x(1),y(1));
handles.axes1.axes_visible = ['off','off','off']; 
handles.axes1.filled = 'on';
handles.axes1.background = -2;
    
subplot(4,1,2);
handles.axes2=gca();
handles.axes2.axes_bounds = [0.52 0 0.52,0.3];
plot(x(1),y(1));
handles.axes2.axes_visible = ['off','off','off']; 

subplot(4,1,3);
handles.axes3=gca(); 
handles.axes3.axes_bounds = [0.52 0.25 0.52,0.3];
plot(x(1),y(1));
handles.axes3.axes_visible = ['off','off','off']; 

subplot(4,1,4);
handles.axes4=gca(); 
handles.axes4.axes_bounds = [0.52 0.5 0.52,0.5];
plot(x(1),y(1));
handles.axes4.axes_visible = ['off','off','off']; 


//*************
// Plant Panel
//*************
uicontrol(h, "units","pixels", "style", "frame","string","Plant","position",[5 465 150 80],'fontsize',12);
  uicontrol(h, "units","pixels", "style", "text","string","Plant","position",[10 520 80 20],'fontsize',12,'FontWeight','bold');
  uicontrol(h, "units","pixels", "style", "text","string","Num","position",[10 500 100 20],'fontsize',12,'HorizontalAlignment','left');
  uicontrol(h, "units","pixels", "style", "text","string","Den","position",[10 470 100 20],'fontsize',12,'HorizontalAlignment','left');
  handles.num= uicontrol(h, "units","pixels", "style", "edit","string",Gnum,"position",[40 500 100 20],'fontsize',12,"Callback","editnum(handles,h)");
  handles.den= uicontrol(h, "units","pixels", "style", "edit","string",Gden,"position",[40 470 100 20],'fontsize',12,"Callback","editden(handles,h)");
  
//*************    
// Control Panel
//*************
handles.update= uicontrol(h, "units","pixels", "style", "pushbutton","string","Update","position",[40 430 100 30],"Callback","pbUpdate(handles,h)",'fontsize',12);
handles.gain= uicontrol(h, "units","pixels", "style", "slider","string","Update","position",[20 400 150 20],"Min",0,"Max",1000,"SliderStep",[1 10],"Callback","sldGain(handles,h)","value",10);
handles.sldval = uicontrol(h, "units","pixels", "style", "text","string",string(10*handles.sldG),"position",[80 380 80 20],'fontsize',12);

//*************    
// Controller Panel
//*************    
uicontrol(h, "units","pixels", "style", "frame","string","Controller","position",[5 300 150 82],'fontsize',12);
  uicontrol(h, "units","pixels", "style", "text","string","Controller","position",[10 360 80 20],'fontsize',12,'FontWeight','bold');
  uicontrol(h, "units","pixels", "style", "text","string","Num","position",[10 340 100 20],'fontsize',12,'HorizontalAlignment','left');
  uicontrol(h, "units","pixels", "style", "text","string","Den","position",[10 310 100 20],'fontsize',12,'HorizontalAlignment','left');
  handles.numC= uicontrol(h, "units","pixels", "style", "edit","string","1","position",[40 340 100 20],'fontsize',12,"Callback","editnumC(handles,h)");
  handles.denC= uicontrol(h, "units","pixels", "style", "edit","string","1","position",[40 310 100 20],'fontsize',12,"Callback","editdenC(handles,h)");
  
//*************    
// Prefilter Panel
//*************    
uicontrol(h, "units","pixels", "style", "frame","string","Prefilter","position",[5 220 150 82],'fontsize',12);
  uicontrol(h, "units","pixels", "style", "text","string","Prefilter","position",[10 280 80 20],'fontsize',12,'FontWeight','bold');
  uicontrol(h, "units","pixels", "style", "text","string","Num","position",[10 260 100 20],'fontsize',12,'HorizontalAlignment','left');
  uicontrol(h, "units","pixels", "style", "text","string","Den","position",[10 230 100 20],'fontsize',12,'HorizontalAlignment','left');
  handles.numF= uicontrol(h, "units","pixels", "style", "edit","string","1","position",[40 260 100 20],'fontsize',12,"Callback","editnumF(handles,h)");
  handles.denF= uicontrol(h, "units","pixels", "style", "edit","string","1","position",[40 230 100 20],'fontsize',12,"Callback","editdenF(handles,h)");

//*************    
// Design Contraint
//*************    
uicontrol(h, "units","pixels", "style", "frame","string","Design Constraint","position",[5 220-120 150 122],'fontsize',12);
  uicontrol(h, "units","pixels", "style", "text","string","Design Constraint","position",[10 280-80 120 20],'fontsize',12,'FontWeight','bold');
  uicontrol(h, "units","pixels", "style", "text","string","%OS","position",[10 260-80 100 20],'fontsize',12,'HorizontalAlignment','left');
  uicontrol(h, "units","pixels", "style", "text","string","Ts","position",[10 230-80 100 20],'fontsize',12,'HorizontalAlignment','left');
  handles.OS = uicontrol(h, "units","pixels", "style", "edit","string","10","position",[40 260-80 100 20],'fontsize',12,"Callback","editOS(handles,h)");
  handles.Ts = uicontrol(h, "units","pixels", "style", "edit","string","1","position",[40 230-80 100 20],'fontsize',12,"Callback","editTS(handles,h)");
  handles.decon= uicontrol(h, "units","pixels", "style", "pushbutton","string","Show Constraint","position",[40 190-80 100 30],"Callback","pbShow(handles,h)",'fontsize',12);

//handles.closef= uicontrol(h, "units","pixels", "style", "pushbutton","string","Close","position",[40 100 100 30],"Callback","pbClose(handles,h)");

// Initialization;s
handles.buffer = 0
handles.axes1_h = 0;
handles.axes2_h = 0;
handles.axes3_h = 0;
handles.axes4_h = 0;
handles.g = 0;

pbUpdate(handles,h)

[handles,h] = resume(handles,h);


endfunction
//*************    
// Update Button
//*************    
function pbUpdate(handles,h)
  
// Preparing handles for plot updates 
    //clear handles.axes1_h; global handles.axes1_h; handles.axes1_h = 0;  
    //clear handles.axes2_h; global handles.axes2_h; handles.axes2_h = 0;   
    //clear handles.axes3_h; global handles.axes3_h; handles.axes3_h = 0;   
    //clear handles.axes4_h; global handles.axes4_h; handles.axes4_h = 0;
    val = get(handles.gain,'value')*handles.sldG;    
    
// Calculation    
 
  s=poly(0,'s');	
  
  // Get the Plant Tranfer Functions
  num = eval(get(handles.num,'string'));
  den = eval(get(handles.den,'string'));
  Gol = syslin('c',(num),den);
  
  // Get the Controller Tranfer Functions
  numC = eval(get(handles.numC,'string'));
  denC = eval(get(handles.denC,'string'));
  Gc=syslin('c',(numC),denC);
  
  numF = eval(get(handles.numF,'string'));
  denF = eval(get(handles.denF,'string'));
  handles.Gf=syslin('c',(numF),denF);

  // Open Loop TF with Controller
  handles.g = Gol * Gc;
  drawlater();
// 1. Root Locus Plot
  sca(handles.axes1);
  if  ~isempty(handles.axes1.children) then
      delete(handles.axes1.children);
  end
  //  Change in 4.0 4/4/2013
  dc = horner(handles.g,0);
  if dc == %inf
    dc = 0;
  end
  
  //evans(handles.g,val*1000);
  //evans(handles.g,1/dc*100); // Change in 4.0 4/4/2013
  evans(handles.g, get(handles.gain,'max')*handles.sldG); // Change in  1/4/2014
  
 // pause
  // Calculate close loop gain
  gl=1+val*handles.g;
  numl=numer(gl);
  r=roots(numl);
  x=real(r);
  y=imag(r);
  plot(x,y,'rx');
  handles.axes1_h = gce();
  
// 2. Bode Mag Plot  
  sca(handles.axes2);
  //wrect=axes.axes_bounds;
  //axes.axes_bounds=[wrect(1)+0,wrect(2)+0,wrect(3)*1.0,wrect(4)*hx*0.95]
  [fdata,ddata,pdata,gdata]=bode2(val*handles.g,0.01,100);
  handles.axes2.data_bounds = [min(fdata),min(ddata);max(fdata),max(ddata)];
  handles.axes2.log_flags = "lnn" ;
  handles.axes2.grid=color('lightgrey')*ones(1,3);
  handles.axes2.axes_visible="on";
  handles.axes2.clip_state = "clipgrf";
  mtlb_grid on;
  if  ~isempty(handles.axes2.children) then
      delete(handles.axes2.children);
  end

  
  xpolys(fdata,ddata,gdata);
  handles.axes2_h =  gce();
  
// 3. Bode Phase Plot  
  sca(handles.axes3);
  //wrect=axes.axes_bounds;
  //axes.axes_bounds=[wrect(1)+0,wrect(2)+0,wrect(3)*1.0,wrect(4)*hx*0.95]
  handles.axes3.data_bounds = [min(fdata),min(pdata);max(fdata),max(pdata)];
  handles.axes3.log_flags = "lnn" ;
  handles.axes3.grid=color('lightgrey')*ones(1,3);
  handles.axes3.axes_visible="on";
  handles.axes3.clip_state = "clipgrf";


  mtlb_grid on;
  if  ~isempty(handles.axes3.children) then
      delete(handles.axes3.children);
  end  
  
  xpolys(fdata,pdata,gdata);
  handles.axes3_h =  gce();
  
// 4. Step Response
  sca(handles.axes4);
  mtlb_grid on;
  if  ~isempty(handles.axes4.children) then
      delete(handles.axes4.children);
  end    
  
  g2 = syslin('c',1,1);
  gcl = ((val*handles.g)/.g2)*handles.Gf ;

//  Change in 4.0 4/4/2013
  //t=0:0.001:10;
  [wn,z,r] = damp(gcl);
  Tset = 4/(min(abs(z))*min(abs(wn)));
  t = linspace(0,ceil(Tset),1000);

  //y=csim('step',t,gcl);  
  gcl2 = tf2ss(gcl);
  y=csim('step',t,gcl2) + gcl2.d;  
  
  plot(t,y);
  mtlb_axis([0 Tset 0 max(y)]);     //  Change in 4.0 4/4/2013
  handles.axes4_h = gce();
  drawnow();
// Resuming Sstates: ??? Is it correct way??? 
 pbShow(handles,h)      // Add in 1/4/2014
[handles] = resume(handles); 

endfunction


function sldGain(handles,h)
 handles.buffer = handles.buffer + 1;  
  if handles.buffer <=1
    
    // Place time consuming code here

    val = get(handles.gain,'value')*handles.sldG;    
    if val <= get(handles.gain,'min')*handles.sldG | val == 0 then
//      if val == 0  then        
        handles.sldG = handles.sldG/10;
        set(handles.gain,'value',10);
        val = get(handles.gain,'value')*handles.sldG;    
        set(handles.sldval,'string',string(val));
        pbUpdate(handles,h)
            
    elseif val == get(handles.gain,'max')*handles.sldG then
        handles.sldG = handles.sldG*10;
        set(handles.gain,'value',100);
        val = get(handles.gain,'value')*handles.sldG;    
        set(handles.sldval,'string',string(val));
        pbUpdate(handles,h)
    end
    
    // Checking Buffer and also Gain
    //disp([handles.buffer,val]);   
    
    //sleep(500);  
    UpdatePlot(handles,h);
    handles.buffer =handles.buffer - 1;
    //set(handles.sldval,'string',string(val));   
    //UpdatePlot(handles,h);
    handles = resume(handles);
    //UpdatePlot(handles,h);
  else
      
    handles.buffer =handles.buffer - 1;
    val = get(handles.gain,'value')*handles.sldG;    
    set(handles.sldval,'string',string(val));
   //disp('.');
    handles = resume(handles);
    
  end  

  handles = resume(handles);
  

  
//try
//   if handles.axes4_h ~= 0 
//    val1 = get(handles.gain,'value');
//    sleep(50);
//    val2 = get(handles.gain,'value');
//    sleep(50);
//    while val1 ~= val2 do
//      //abort;
//      disp('Moving');
//   end
//    
//    
//    
//  end
//catch
//  //disp(lasterror);
//  //abort;
//  disp('Error');
//end
////UpdatePlot(handles,h)
////sldGain(handles,h)
// aaa = gcb

endfunction



function UpdatePlot(handles,h)
  
    val = get(handles.gain,'value')*handles.sldG;    
    set(handles.sldval,'string',string(val));
    
    gl=1+val*handles.g;
    numl=numer(gl);
    r=roots(numl);
    x=real(r);
    y=imag(r);
    
    g2 = syslin('c',1,1);
    gcl = ((val*handles.g)/.g2)*handles.Gf;

    //  Change in 4.0 4/4/2013
    
    //    t=0:0.001:10;
    [wn,z,r] = damp(gcl);
    //Tset = 4/(min(z(z>0))*min(wn(wn>0)));
    Tset = 4/(min(abs(z))*min(abs(wn)));    
    t = linspace(0,ceil(Tset),1000);

    //y2=csim('step',t,gcl);  
    gcl2 = tf2ss(gcl);
    y2=csim('step',t,gcl2) + gcl2.d;  
  
    [fdata,ddata,pdata,gdata]=bode2(val*handles.g,0.01,100);
    
    drawlater();
    // 1. Root Locus
    sca(handles.axes1);
    set(handles.axes1_h.children,'data',[x,y]);
    
    // 2. Bode Mag Plot  
    sca(handles.axes2);
    mtlb_grid on;
    set(handles.axes2_h.children,'data',[fdata,ddata]);
    handles.axes2.data_bounds = [min(fdata),min(ddata);max(fdata),max(ddata)];
    
    // 3. Phase Mag Plot  
    sca(handles.axes3);
    mtlb_grid on;
    set(handles.axes3_h.children,'data',[fdata,pdata]);
    handles.axes3.data_bounds = [min(fdata),min(pdata);max(fdata),max(pdata)];
        
    // 4. Step Response
    sca(handles.axes4);  
    mtlb_axis([0 Tset min(y2) max(y2)]);   //  Change in 4.0 4/4/2013
    set(handles.axes4_h.children,'data',[t',y2']);

if handles.buffer <=1
    drawnow();
end
   [handles] = resume(handles);
   
endfunction

function pbShow(handles,h)
  sca(handles.axes1);
  OS = eval(get(handles.OS,'string'));  
  zeta = abs(log(OS/100))/(sqrt(%pi^2 + log(OS/100)^2));
  Ts = eval(get(handles.Ts,'string'));  
  wn = 4/(zeta*Ts);
  sgrid(zeta,wn);
 // pbUpdate(handles,h)
 //   [handles] = resume(handles);

endfunction

//////////////////////////////////////////////////////////
// Edit Boxes Functions (Added on 1/4/2014)
//////////////////////////////////////////////////////////

function editnumC(handles,h)
    s = %s;
    try numC = eval(get(handles.numC,'string'))
        if  type(numC) ~=2 & numC <= 0 then
            messagebox('Controller numerator must be greater than 0 or a valid polynomial','Error','error');
            set(handles.numC,'string','1');
        end
    catch
        messagebox('Controller numerator must be greater than 0 or a valid polynomial','Error','error');
        set(handles.numC,'string','1');
    end
    pbUpdate(handles,h)
    [handles] = resume(handles);
endfunction

function editdenC(handles,h)
    s = %s;
    try denC = eval(get(handles.denC,'string'))
        if  type(denC) ~=2 & denC <= 0 then
            messagebox('Controller denominator must be greater than 0 or a valid polynomial','Error','error');
            set(handles.denC,'string','1');
        end
    catch
        messagebox('Controller denominator must be greater than 0 or a valid polynomial','Error','error');
        set(handles.denC,'string','1');
    end
    pbUpdate(handles,h)
    [handles] = resume(handles);
endfunction

function editnum(handles,h)
    s = %s;
    try num = eval(get(handles.num,'string'))
        if  type(num) ~=2 & num <= 0 then
            messagebox('Plant numerator must be greater than 0 or a valid polynomial','Error','error');
            set(handles.num,'string','1');
        end
    catch
        messagebox('Plant numerator must be greater than 0 or a valid polynomial','Error','error');
        set(handles.num,'string','1');
    end
    pbUpdate(handles,h)
    [handles] = resume(handles);
endfunction

function editden(handles,h)
    s = %s;
    try den = eval(get(handles.den,'string'))
        if  type(den) ~=2 & den <= 0 then
            messagebox('Plant denominator must be greater than 0 or a valid polynomial','Error','error');
            set(handles.den,'string','1');
        end
    catch
        messagebox('Plant denominator must be greater than 0 or a valid polynomial','Error','error');
        set(handles.den,'string','1');
    end
    pbUpdate(handles,h)
    [handles] = resume(handles);
endfunction

function editnumF(handles,h)
    s = %s;
    try numF = eval(get(handles.numF,'string'))
        if  type(numF) ~=2 & numF <= 0 then
            messagebox('Pre-filter numerator must be greater than 0 or a valid polynomial','Error','error');
            set(handles.numF,'string','1');
        end
    catch
        messagebox('Pre-filter numerator must be greater than 0 or a valid polynomial','Error','error');
        set(handles.numF,'string','1');
    end
    pbUpdate(handles,h)
    [handles] = resume(handles);
endfunction

function editdenF(handles,h)
    s = %s;
    try denF = eval(get(handles.denF,'string'))
        if  type(denF) ~=2 & denF <= 0 then
            messagebox('Pre-filter denominator must be greater than 0 or a valid polynomial','Error','error');
            set(handles.denF,'string','1');
        end
    catch
        messagebox('Pre-filter denominator must be greater than 0 or a valid polynomial','Error','error');
        set(handles.denF,'string','1');
    end
    pbUpdate(handles,h)
    [handles] = resume(handles);
endfunction

function editOS(handles,h)
    //s = %s;
    try OS = eval(get(handles.OS,'string'))
        //pause
        if   OS <= 0 | OS >= 100  then
            messagebox('Overshoot must be > 0 and < 100','Error','error');
            set(handles.OS,'string','10');
        end
    catch
        messagebox('Overshoot must be > 0 and < 100','Error','error');
        set(handles.OS,'string','10');
    end
    pbUpdate(handles,h)
    [handles] = resume(handles);
endfunction

function editTS(handles,h)
    s = %s;
    try Ts = eval(get(handles.Ts,'string'))
        if  Ts <= 0 then
            messagebox('Settling time must be > 0','Error','error');
            set(handles.Ts,'string','1');
        end
    catch
            messagebox('Settling time must be > 0','Error','error');
        set(handles.Ts,'string','1');
    end
   pbUpdate(handles,h)
    [handles] = resume(handles);
endfunction

// Do note that bode2 is mmodified from the Scilab Function "bode"
// Scilab ( http://www.scilab.org/ ) - This file is part of Scilab
// Copyright (C) INRIA Serge Steer
// This file must be used under the terms of the CeCILL.
// This source file is licensed as described in the file COPYING, which
// you should have received as part of this distribution.  The terms
// are also available at    
// http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt

function [fdata,ddata,pdata,gdata]=bode2(varargin)
  rhs=size(varargin)
  if type(varargin($))==10 then
    comments=varargin($),rhs=rhs-1;
  else
    comments=[];
  end
  fname="bode";//for error messages
  fmax=[]
  if or(typeof(varargin(1))==['state-space' 'rational']) then 
    //sys,fmin,fmax [,pas] or sys,frq
    refdim=1 //for error message
    if rhs==1 then //sys
      [frq,repf]=repfreq(varargin(1),1d-3,1d3)
    elseif rhs==2 then //sys,frq
      if size(varargin(2),2)<2 then
	error(msprintf(_("%s: Wrong size for input argument #%d: A row vector with length>%d expected.\n"),..
		       fname,2,1))
      end
      [frq,repf]=repfreq(varargin(1:rhs))
    elseif or(rhs==(3:4)) then //sys,fmin,fmax [,pas]
      [frq,repf]=repfreq(varargin(1:rhs))
    else
      error(msprintf(_("%s: Wrong number of input arguments: %d to %d expected.\n"),fname,1,5))
    end
    [phi,d]=phasemag(repf)
    if rhs>=3 then fmax=varargin(3),end
  elseif  type(varargin(1))==1 then 
    //frq,db,phi [,comments] or frq, repf [,comments]
    refdim=2
    select rhs
    case 2 then //frq,repf
      frq=varargin(1);
      if size(frq,2)<2 then
	error(msprintf(_("%s: Wrong size for input argument #%d: A row vector with length>%d expected.\n"),..
		       fname,1,1))
      end
      if size(frq,2)<>size(varargin(2),2) then
	error(msprintf(_("%s: Incompatible input arguments #%d and #%d: Same column dimensions expected.\n"),..
			 fname,1,2))
      end
      [phi,d]=phasemag(varargin(2))
    case 3 then  //frq,db,phi
      [frq,d,phi]=varargin(1:rhs)
      if size(frq,2)<>size(d,2) then
	error(msprintf(_("%s: Incompatible input arguments #%d and #%d: Same column dimensions expected.\n"),..
			 fname,1,2))
      end
      if size(frq,2)<>size(phi,2) then
	error(msprintf(_("%s: Incompatible input arguments #%d and #%d: Same column dimensions expected.\n"),..
			 fname,1,3))
      end
    else 
       error(msprintf(_("%s: Wrong number of input arguments: %d to %d expected.\n"),fname,2,4))
    end
  else
    error(msprintf(_("%s: Wrong type for input argument #%d: Linear dynamical system or row vector of floats expected.\n"),fname,1))
  end;
   frq=frq';d=d',phi=phi'
  [n,mn]=size(d)

  if comments==[] then
    hx=0.48
  else
    if size(comments,'*')<>mn then
      error(msprintf(_("%s: Incompatible input arguments #%d and #%d: Same number of elements expected.\n"),...
		     fname,refdim,rhs+1))
    end
    hx=0.43
  end;

  if size(d,2)>1&size(frq,2)==1 then
    fdata = frq(:,ones(1,mn));
    ddata = d;
    gdata = 1:mn;
  else
    fdata = frq;
    ddata = d;
    gdata = 1:mn;
  end
 
  if size(phi,2)>1&size(frq,2)==1 then
    pdata = phi;
  else
    pdata = phi;
  end


endfunction

//function [gain] = dcgain(g)

//g2 = tf2ss(g);
//gain = g2.D - g2.C*inv(g2.A)*g2.B;

//endfunction
